import java.util.Objects;

/**
 * Encapsulates velocity data for a Car
 */
public class Car implements Comparable<Car> {

    /** Current speed of the car in mph */
    private int speed;
    /** Maximum possible speed of the car in mph */
    private final int maxSpeed;

    /**
     * Create a new instance of a Car object.
     * 
     * @param maxSpeed indicates the maximum possible speed of the car in mph
     * @throws IllegalArgumentException if the given maxSpeed is negative
     */
    public Car(int maxSpeed) {
        if (maxSpeed < 0)
            throw new IllegalArgumentException("Given maximum speed must not be negative but is: " + maxSpeed);
        this.maxSpeed = maxSpeed;
    }

    /**
     * Accessor method for the current speed of the car.
     * 
     * @return current speed of the car in mph
     */
    public int getSpeed() {
        return speed;
    }

    /**
     * Increase or decrease the current speed of the car.
     * 
     * The speed will never fall below zero or go beyond the specified maximum
     * speed.
     * 
     * @param delta delta value to be applied in mph.
     */
    public void gas(int delta) {
        speed += delta;
        if (speed > maxSpeed)
            speed = maxSpeed;
        if (speed < 0)
            speed = 0;
    }

    /**
     * Compares the speed of the given Car instance to this instance's speed.
     * 
     * @param that the car to be compared.
     * @return a negative integer, zero, or a positive integer as this car's speed
     *         is less than, equal to, or greater than the given object's speed.
     * @throws NullPointerException if the given car parameter is null
     */
    @Override
    public int compareTo(Car that) {
        Objects.requireNonNull(that, "The given Car instance must not be null.");

        int result;
        if (that.speed < this.speed)
            result = 1;
        else if (that.speed > this.speed)
            result = -1;
        else
            result = 0;

        // alternatively, you could use the static Integer comparison method:
        // result = Integer.compare(this.speed, that.speed);

        return result;
    }
}
